﻿// © 2008 Michele Leroux Bustamante. All rights reserved 
// Book: Learning WCF, O'Reilly
// Book Blog: www.thatindigogirl.com
// Michele's Blog: www.dasblonde.net
// IDesign: www.idesign.net

using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Claims;
using System.Security.Cryptography.X509Certificates;
using WCFClaims = System.IdentityModel.Claims;
using Microsoft.IdentityModel.SecurityTokenService;
using System.IdentityModel.Tokens;


namespace STS
{
    public class IdentitySTS : SecurityTokenService
    {
        public IdentitySTS(SecurityTokenServiceConfiguration config)
            : base( config )
        {
        }

        protected override IClaimsIdentity GetOutputClaimsIdentity(IClaimsPrincipal principal, RequestSecurityToken request, Scope scope)
        {
            IClaimsIdentity claimsIdentity = new ClaimsIdentity();

            // TODO: look up Windows account and map to claims
            // this example hard codes a specific list of claims for any user identity
            claimsIdentity.Claims.Add(new Claim(WCFClaims.ClaimTypes.Name, principal.Identity.Name));
            claimsIdentity.Claims.Add(new Claim(Constants.ClaimTypes.Permission, Constants.Permissions.Create));
            claimsIdentity.Claims.Add(new Claim(Constants.ClaimTypes.Permission, Constants.Permissions.Read));
            claimsIdentity.Claims.Add(new Claim(Constants.ClaimTypes.Permission, Constants.Permissions.Update));

            return claimsIdentity;
        }

        protected override Scope  GetScope(Microsoft.IdentityModel.Claims.IClaimsPrincipal principal, RequestSecurityToken request)
        {

            Scope scope = new Scope(request);
            scope.EncryptingCredentials = this.GetCredentialsForAppliesTo(request.AppliesTo);
            scope.SigningCredentials = new X509SigningCredentials(CertificateUtil.GetCertificate(StoreName.My, StoreLocation.LocalMachine, "CN=IPKey"));

            return scope;
        }

        private X509EncryptingCredentials GetCredentialsForAppliesTo(EndpointAddress appliesTo)
        {
            if (appliesTo == null || appliesTo.Uri ==null || string.IsNullOrEmpty(appliesTo.Uri.AbsolutePath))
            {
                throw new InvalidRequestException("AppliesTo must be supplied in the RST.");
            }

            X509EncryptingCredentials creds = null;
            if (appliesTo.Uri.AbsoluteUri.StartsWith("http://localhost:49546/RPHostWeb/CrudService.svc"))
            {
                creds = new X509EncryptingCredentials(CertificateUtil.GetCertificate(StoreName.TrustedPeople, StoreLocation.LocalMachine, "CN=RPKey"));
            }
            else
                throw new InvalidRequestException(String.Format("Invalid relying party address: {0}", appliesTo.Uri.AbsoluteUri));

            return creds;
        }
    }
}
